#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <assert.h>
#include <sys/cdrom.h>
#include <sys/qioctl.h>
#ifdef __QNXNTO__
#include <sys/dcmd_blk.h>
#endif

#include "proto.h"
#include "cd.h"

const char *prompt = "cdp>";
extern cdinfo_t			cdinfo;

void DrawPrompt()
{
	cprintf("\n\r%s", prompt );
}

void Help()
{
	cprintf(" \n\r console cdplayer v1.0 coded by ccpro\n\r");
	cprintf(" [h]elp\n\r"
			" [p]lay\n\r"
			" [s]top\n\r"
			" [>]next\n\r"
			" [<]next\n\r"
			" p[a]use\n\r"
			" [r]esume\n\r"
			" [e]ject\n\r"
			" [i]nfo\n\r"
			" [q]uit\n\r"
			);	
}

void PlayCD()
{
	if( cdinfo.mode == M_PLAY )
		return;
	if( cdinfo.atrack >= cdinfo.adisc->tochdr.cdth_trk0 &&
		cdinfo.atrack <= cdinfo.adisc->tochdr.cdth_trk1  )	
		cd_play( 0 );
	else 													
		cd_play( 1 );
}

void StopCD()
{
	cd_stop();
	cdinfo.atrack = 0;
}

void NextPrevCD( int next )
{
	ulong_t		slba, elba;

	if( cdinfo.atrack == -1 ) 
	{
		cdinfo.atrack = cdinfo.adisc->tochdr.cdth_trk0;
	} else 
	{
		cdinfo.atrack += next ;
		if( cdinfo.atrack < cdinfo.adisc->tochdr.cdth_trk0 &&
			cdinfo.atrack > cdinfo.adisc->tochdr.cdth_trk1 ) 
			cdinfo.atrack = 0;
	}
	if( cdinfo.atrack > cdinfo.adisc->tochdr.cdth_trk1  ) 
	{
		cdinfo.adisc = cdinfo.adisc->nxt;
		if( cdinfo.adisc == NULL ) 
		{
			cdinfo.adisc = cdinfo.disc_list;
		}
		cd_change_disc( cdinfo.adisc->dnum );
		cdinfo.atrack = cdinfo.adisc->tochdr.cdth_trk0;
		if( !cdinfo.continuous ) 
		{
			return;
		}
	}
	slba = CD_TOCENTRY( cdinfo.atrack )->cdte_addr.lba;
	elba = CD_TOCENTRY( cdinfo.atrack + 1 )->cdte_addr.lba;
	if( cdinfo.intro ) 
	{
		elba = slba + 1000;		// intro duration
	}
	cd_play_audio( slba, elba );
	InfoCD();
}


void PauseCD()
{
	cd_pause_resume();
}

void ResumeCD()
{
	cd_pause_resume();
}

void EjectCD()
{
	cd_eject_load( -1 );
}

void InfoCD()
{
	cprintf( "CD <%2.2d-%2.2d:%2.2d>", cdinfo.atrack, cdinfo.reladdr.msf.minute, cdinfo.reladdr.msf.second );
}

void Quit()
{
}

int CommandLoop()
{
	char buf[256];
	DrawPrompt();
	
	if( cdinfo.mode == M_NODISC ) 
	{
		display_NODISC();
		cd_check_discs();
		return 1;
	}else if( cdinfo.mode == M_DATA ) 
	{
		display_DATA();
		cd_check_discs();
		return 1;
	}
	gets( buf );
	
	switch( tolower(buf[0]) )
	{
		case 'p' :
			PlayCD();
			break;
	
		case 's' :
			StopCD();
			break;
	
		case 'a' :
			PauseCD();
			break;
	
		case 'r' :
			ResumeCD();
			break;
	
		case 'e' :
			EjectCD();
			break;
	
		case 'i' :
			InfoCD();
			break;

		case '>' :
			NextPrevCD( 1 ); // next
			break;

		case '<' :
			NextPrevCD( -1 ); // prev
			break;

		case 'q' :
			Quit();
			return 0;
			break;
	
		case  'h':
		default: Help();
	}

	return 1;
}

int main( int argc, char **argv )
{	
	Help();
	
	startup();
	cfg_init();

	while( CommandLoop());

	return 0;
}
